// ======================== Certis (C++) Libs ================================
//       LevelSets Library
// ===========================================================================
// Copyright (C) 2000-2007 Certis/Ecole des Ponts and Contributors
// For detailed information: http://www.enpc.fr/certis/software
// ===========================================================================

#ifndef _CLLEVELSETSFLUXES_H
#define _CLLEVELSETSFLUXES_H

#include <Imagine/Images.h>

namespace Imagine {

	const double DELTA = 1.;

	template <class approximator, typename T, int dim>
	T advection(const Image<T,dim>& u, const Coords<dim>& p, const FVector<T,dim>& v) {
		T delta = 0;
		for (int i=0;i<dim;i++) {
			delta -= v[i] * ( (v[i]>0) ? approximator::u_mi(u,p,i) : approximator::u_pi(u,p,i) );
		}
		return delta;
	}


	template <class approximator, typename T, int dim>
	T normal_motion(const Image<T,dim>& u, const Coords<dim>& p, const T beta) {

		T delta = 0;
		for (int i=0;i<dim;i++) {
			const T upi = approximator::u_pi(u,p,i);
			const T umi = approximator::u_mi(u,p,i);

			if (beta > 0) {
				if (upi < 0) delta += upi*upi;
				if (umi > 0) delta += umi*umi;
			}
			else {
				if (upi > 0) delta += upi*upi;
				if (umi < 0) delta += umi*umi;
			}
		}

		return - beta * std::sqrt(delta);
	}


	template <class approximator, typename T, typename U, int dim>
	T extension(const Image<T,dim>& u, const Image<U,dim>& d, const Coords<dim>& p) {

		FVector<T,dim> v;
		normal(u,p,v);
		const T u0 = u(p);
		const T s = u0 / sqrt( u0*u0 + T(DELTA)*T(DELTA) );
		v *= s;
		return advection<approximator>(d,p,v);
	}


	template <class approximator, typename T, int dim>
	T reinitialization(const Image<T,dim>& u, const Coords<dim>& p) {

		const T u0 = u(p);
		const T s = u0 / std::sqrt( u0*u0 + T(DELTA)*T(DELTA) );
		return s + normal_motion<approximator>(u,p,s);
	}
}


#endif
