// ======================== Certis (C++) Libs ================================
//       LevelSets Library
// ===========================================================================
// Copyright (C) 2000-2007 Certis/Ecole des Ponts and Contributors
// For detailed information: http://www.enpc.fr/certis/software
// ===========================================================================

#ifndef _CLLEVELSETSIO_H
#define _CLLEVELSETSIO_H

#include <vector>

#include "MarchingCubes.h"
#include "MarchingSquares.h"

#include <Imagine/Images.h>

namespace Imagine {
	template <typename T>
	void extract_isosurface(const Image<T,3>& phi, T value, MeshVTK& M, const Affine<T,3>& affine = Affine<T,3>(Identity()))
	{
		typedef FVector<T,3> Vector3;
		std::vector<Vector3> points,normals;
		std::vector<Triangle> triangles;
		marching_cubes(phi, value, std::back_inserter(points), std::back_inserter(normals), affine);

		for (int i=0;i<int(points.size())/3;i++) {
			triangles.push_back( Triangle( 3*i, 3*i+1, 3*i+2) );
		}
		M = MeshVTK(&(points[0]), int(points.size()), &(triangles[0]), int(triangles.size()));
	}

	template <typename T>
	void extract_zero_level(const Image<T,3>& phi, MeshVTK& M, const Affine<T,3>& affine = Affine<T,3>(Identity()))
	{
		return extract_isosurface(phi,T(0),M,affine);
	}

	template <typename T>
	void display_isocontour(const Image<T,2>& phi, T value, Color col, const Affine<T,2>& affine = Affine<T,2>(Identity()))
	{
		typedef FVector<int,2> Point2;
		std::vector<Point2> points,normals;
		marching_squares(phi, value, std::back_inserter(points), std::back_inserter(normals), affine);
		
		NoRefreshBegin();
		for (int i=0;i<int(points.size())/2;i++)
			DrawLine(points[2*i],points[2*i+1],col);
		NoRefreshEnd();
	}

	template <typename T>
	void display_zero_level(const Image<T,2>& phi, Color col, const Affine<T,2>& affine = Affine<T,2>(Identity()))
	{
		return display_isocontour(phi,T(0),col,affine);
	}
} 
#endif
