// ===========================================================================
// Imagine++ Libraries
// Copyright (C) Imagine
// For detailed information: http://imagine.enpc.fr/software
// ===========================================================================


namespace Imagine {
/// \addtogroup LevelSets
	/// @{

	
	const double FLUX_DELTA = 1.;

	template <class approximator, typename T, int dim>
	T advection(const Image<T,dim>& u, const Coords<dim>& p, const FVector<T,dim>& v) {
		T delta = 0;
		for (int i=0;i<dim;i++) {
			delta -= v[i] * ( (v[i]>0) ? approximator::u_mi(u,p,i) : approximator::u_pi(u,p,i) );
		}
		return delta;
	}


	template <class approximator, typename T, int dim>
	T normalMotion(const Image<T,dim>& u, const Coords<dim>& p, const T beta) {

		T delta = 0;
		for (int i=0;i<dim;i++) {
			const T upi = approximator::u_pi(u,p,i);
			const T umi = approximator::u_mi(u,p,i);

			if (beta > 0) {
				if (upi < 0) delta += upi*upi;
				if (umi > 0) delta += umi*umi;
			}
			else {
				if (upi > 0) delta += upi*upi;
				if (umi < 0) delta += umi*umi;
			}
		}

		return - beta * std::sqrt(delta);
	}


	template <class approximator, typename T, typename U, int dim>
	T extension(const Image<T,dim>& u, const Image<U,dim>& d, const Coords<dim>& p) {

		FVector<T,dim> v;
		normal(u,p,v);
		const T u0 = u(p);
		const T s = u0 / sqrt( u0*u0 + T(FLUX_DELTA)*T(FLUX_DELTA) );
		v *= s;
		return advection<approximator>(d,p,v);
	}


	template <class approximator, typename T, int dim>
	T reinitialization(const Image<T,dim>& u, const Coords<dim>& p) {

		const T u0 = u(p);
		const T s = u0 / std::sqrt( u0*u0 + T(FLUX_DELTA)*T(FLUX_DELTA) );
		return s + normalMotion<approximator>(u,p,s);
	}
	///@}
}
