// ===========================================================================
// Imagine++ Libraries
// Copyright (C) Imagine
// For detailed information: http://imagine.enpc.fr/software
// ===========================================================================

namespace Imagine {
	/// \addtogroup LevelSets
	/// @{

	template <typename T>
	void extractIsoSurface(const Image<T,3>& phi, T value, VtkMesh& M, const Affine<T,3>& affine = Affine<T,3>::Identity())
	{
		typedef FVector<T,3> Vector3;
		std::vector<Vector3> points,normals;
		std::vector<Triangle> triangles;
		marchingCubes(phi, value, std::back_inserter(points), std::back_inserter(normals), affine);

		for (int i=0;i<int(points.size())/3;i++) {
			triangles.push_back( Triangle( 3*i, 3*i+1, 3*i+2) );
		}
		M = VtkMesh(&(points[0]), int(points.size()), &(triangles[0]), int(triangles.size()));
	}

	template <typename T>
	void extractZeroLevel(const Image<T,3>& phi, VtkMesh& M, const Affine<T,3>& affine = Affine<T,3>::Identity())
	{
		return extractIsoSurface(phi,T(0),M,affine);
	}

	template <typename T>
	void displayIsoContour(const Image<T,2>& phi, T value, Color col, const Affine<T,2>& affine = Affine<T,2>::Identity())
	{
		typedef FVector<int,2> Point2;
		std::vector<Point2> points,normals;
		marchingSquares(phi, value, std::back_inserter(points), std::back_inserter(normals), affine);

		noRefreshBegin();
		for (int i=0;i<int(points.size())/2;i++)
			drawLine(points[2*i],points[2*i+1],col);
		noRefreshEnd();
	}

	template <typename T>
	void displayZeroLevel(const Image<T,2>& phi, Color col, const Affine<T,2>& affine = Affine<T,2>::Identity())
	{
		return displayIsoContour(phi,T(0),col,affine);
	}
	///@}
} 
