// ===========================================================================
// Imagine++ Libraries
// Copyright (C) Imagine
// For detailed information: http://imagine.enpc.fr/software
// ===========================================================================

namespace Imagine {
	/// \addtogroup LevelSets
	/// @{

	// First-order one-step Euler time integrator
	template <typename T, int dim>
	class Euler : public Image<T,dim> {

	protected:
		Image<T,dim>& I;

	public:
		Euler(Image<T,dim> &_I) : Image<T,dim>(_I.sizes()), I(_I) {}

		template <class InputIterator>
		bool step(InputIterator first, InputIterator last, T dt) {

			for (InputIterator p = first ; p != last ; ++p) {
				const size_t o = I.offset(*p);
				I[o] += dt * (*this)[o];
			}

			return true;
		}
	};


	// Midpoint time integrator
	template <typename T, int dim>
	class Midpoint : public Image<T,dim> {

	protected:
		Image<T,dim>& I;
		Image<T,dim> tmp;
		int substep;

	public:
		Midpoint(Image<T,dim> &_I) : Image<T,dim>(_I.dims()), I(_I), tmp(_I.dims()), substep(0) {}

		template <class InputIterator>
		bool step(InputIterator first, InputIterator last, T dt) {

			// First substep
			if (substep == 0) {
				for (InputIterator p = first ; p != last ; ++p) {
					const int o = I.offset(*p);
					tmp[o] = I[o];
					I[o] += (dt / T(2)) * (*this)[o];
				}
				substep++;
				return false;
			}

			// Second substep
			for (InputIterator p = first ; p != last ; ++p) {
				const int o = I.offset(*p);
				I[o] = tmp[o] + dt * (*this)[o];
			}
			substep = 0;
			return true;
		}
	};

	// TODO : general Runge-Kutta integrators
	///@}
}
