// ===========================================================================
// Imagine++ Libraries
// Copyright (C) Imagine
// For detailed information: http://imagine.enpc.fr/software
// ===========================================================================

namespace Imagine {
	/// \addtogroup LevelSets
	/// @{

	
	// Structure decrivant un cas de marching cubes
	struct McCase {
		int edges[19];
	};

	// Correspondance entre les aretes et les points
	const int MC_EDGES_TABLE[12][2] = {
		{0,1}, {1,2}, {3,2}, {0,3},
		{4,5}, {5,6}, {7,6}, {4,7},
		{0,4}, {1,5}, {3,7}, {2,6}
	};

	// Table des cas
	const McCase MC_CASES_TABLE[] = {
		{{-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 0, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 9, 3, 3, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 2, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 3, 8, 2, 1, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 0, 11, 11, 0, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 9, 11, 8, 11, 2, 8, 2, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 2, 8, 8, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 0, 9, 3, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 8, 9, 10, 9, 1, 10, 1, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 10, 1, 1, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 10, 8, 11, 8, 0, 11, 0, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 11, 10, 9, 10, 3, 9, 3, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 9, 10, 10, 9, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 4, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 7, 0, 0, 7, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 8, 7, 0, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 3, 7, 1, 7, 4, 1, 4, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 4, 8, 1, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 0, 7, 7, 0, 3, 2, 1, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 0, 11, 11, 0, 9, 4, 8, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 4, 3, 4, 9, 3, 3, 9, 2, 2, 9, 11, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 7, 4, 10, 3, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 0, 2, 4, 2, 10, 4, 10, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 0, 9, 7, 4, 8, 10, 3, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 4, 10, 4, 2, 10, 4, 9, 2, 2, 9, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 10, 1, 1, 10, 3, 8, 7, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 11, 10, 7, 4, 11, 4, 1, 11, 4, 0, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 7, 4, 0, 10, 3, 0, 9, 10, 10, 9, 11, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 11, 10, 4, 9, 10, 7, 4, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 9, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 3, 9, 4, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 1, 4, 4, 1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 1, 3, 5, 3, 8, 5, 8, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 9, 4, 1, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 1, 11, 4, 5, 9, 8, 0, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 0, 4, 2, 4, 5, 2, 5, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 5, 8, 5, 3, 8, 5, 11, 3, 3, 11, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 9, 4, 3, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 8, 2, 2, 8, 0, 9, 4, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 1, 4, 4, 1, 0, 3, 2, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 10, 8, 4, 5, 10, 5, 2, 10, 5, 1, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 1, 10, 10, 1, 11, 5, 9, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 4, 5, 1, 8, 0, 1, 11, 8, 8, 11, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 5, 0, 5, 11, 0, 0, 11, 3, 3, 11, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 10, 8, 5, 11, 8, 4, 5, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 5, 8, 8, 5, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 7, 5, 3, 5, 9, 3, 9, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 5, 1, 7, 1, 0, 7, 0, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 1, 7, 7, 1, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 5, 8, 8, 5, 9, 1, 11, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 11, 2, 0, 5, 9, 0, 3, 5, 5, 3, 7, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 7, 5, 11, 2, 7, 2, 8, 7, 2, 0, 8, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 7, 5, 2, 3, 5, 11, 2, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 8, 5, 5, 8, 7, 10, 3, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 9, 7, 9, 0, 7, 7, 0, 10, 10, 0, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 2, 10, 8, 1, 0, 8, 7, 1, 1, 7, 5, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 5, 1, 10, 7, 1, 2, 10, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 8, 7, 5, 9, 8, 11, 10, 3, 11, 3, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 0, 10, 0, 11, 10, 0, 1, 11, 5, 0, 7, 9, 0, 5, -1, -1, -1, -1}}, 
		{{11, 0, 5, 0, 7, 5, 0, 8, 7, 10, 0, 11, 3, 0, 10, -1, -1, -1, -1}}, 
		{{7, 11, 10, 5, 11, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 3, 11, 5, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 5, 6, 9, 1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 9, 3, 3, 9, 1, 11, 5, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 2, 5, 5, 2, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 2, 5, 5, 2, 1, 0, 3, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 2, 0, 6, 0, 9, 6, 9, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 8, 9, 5, 6, 8, 6, 3, 8, 6, 2, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 5, 2, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 2, 8, 8, 2, 10, 6, 11, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 2, 10, 5, 6, 11, 9, 1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 5, 6, 2, 9, 1, 2, 10, 9, 9, 10, 8, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 1, 5, 3, 5, 6, 3, 6, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 10, 0, 1, 10, 10, 1, 6, 6, 1, 5, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 6, 9, 6, 0, 9, 6, 10, 0, 0, 10, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 8, 9, 6, 10, 9, 5, 6, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 4, 8, 5, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 7, 0, 0, 7, 4, 5, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 5, 6, 8, 7, 4, 0, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 6, 11, 9, 7, 4, 9, 1, 7, 7, 1, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 5, 2, 2, 5, 6, 7, 4, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 7, 4, 0, 3, 7, 1, 5, 6, 1, 6, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 8, 7, 5, 0, 9, 5, 6, 0, 0, 6, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 9, 2, 9, 6, 2, 9, 5, 6, 7, 9, 3, 4, 9, 7, -1, -1, -1, -1}}, 
		{{2, 10, 3, 4, 8, 7, 5, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 5, 7, 2, 10, 7, 4, 2, 2, 4, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 8, 7, 9, 1, 0, 5, 6, 11, 10, 3, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 5, 10, 7, 4, 2, 10, 4, 9, 2, 4, 1, 2, 9, -1, -1, -1, -1}}, 
		{{7, 4, 8, 10, 5, 6, 10, 3, 5, 5, 3, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 10, 0, 10, 4, 0, 10, 7, 4, 5, 10, 1, 6, 10, 5, -1, -1, -1, -1}}, 
		{{4, 8, 7, 9, 5, 6, 0, 9, 6, 10, 0, 6, 3, 0, 10, -1, -1, -1, -1}}, 
		{{7, 4, 9, 10, 7, 9, 10, 9, 5, 10, 5, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 6, 9, 9, 6, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 9, 6, 6, 9, 4, 8, 0, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 4, 6, 0, 6, 11, 0, 11, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 4, 11, 1, 4, 4, 1, 8, 8, 1, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 6, 2, 4, 2, 1, 4, 1, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 3, 8, 9, 2, 1, 9, 4, 2, 2, 4, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 6, 0, 0, 6, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 6, 2, 8, 4, 2, 3, 8, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 6, 9, 9, 6, 11, 2, 10, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 6, 11, 9, 4, 6, 0, 2, 10, 0, 10, 8, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 10, 3, 1, 6, 11, 1, 0, 6, 6, 0, 4, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 1, 8, 1, 10, 8, 1, 2, 10, 6, 1, 4, 11, 1, 6, -1, -1, -1, -1}}, 
		{{10, 4, 6, 10, 3, 4, 3, 9, 4, 3, 1, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 1, 6, 1, 4, 6, 1, 9, 4, 8, 1, 10, 0, 1, 8, -1, -1, -1, -1}}, 
		{{0, 4, 6, 3, 0, 6, 10, 3, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 10, 8, 6, 10, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 9, 8, 11, 8, 7, 11, 7, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 3, 7, 6, 11, 3, 11, 0, 3, 11, 9, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 11, 7, 11, 8, 7, 11, 1, 8, 8, 1, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 3, 7, 11, 1, 7, 6, 11, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 7, 9, 7, 6, 9, 9, 6, 1, 1, 6, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 9, 7, 9, 3, 7, 9, 0, 3, 2, 9, 6, 1, 9, 2, -1, -1, -1, -1}}, 
		{{6, 2, 0, 7, 6, 0, 8, 7, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 3, 7, 2, 3, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 3, 2, 6, 8, 7, 6, 11, 8, 8, 11, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 7, 9, 7, 11, 9, 7, 6, 11, 2, 7, 0, 10, 7, 2, -1, -1, -1, -1}}, 
		{{10, 3, 2, 7, 6, 11, 8, 7, 11, 1, 8, 11, 0, 8, 1, -1, -1, -1, -1}}, 
		{{6, 11, 1, 7, 6, 1, 7, 1, 2, 7, 2, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 6, 1, 6, 3, 1, 6, 10, 3, 8, 6, 9, 7, 6, 8, -1, -1, -1, -1}}, 
		{{6, 10, 7, 9, 0, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 7, 6, 0, 8, 6, 0, 6, 10, 0, 10, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 10, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 6, 3, 8, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 6, 1, 0, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 3, 9, 9, 3, 8, 7, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 2, 1, 10, 6, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 3, 8, 6, 7, 10, 11, 2, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 11, 0, 0, 11, 2, 10, 6, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 6, 7, 3, 11, 2, 3, 8, 11, 11, 8, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 3, 6, 6, 3, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 2, 6, 0, 6, 7, 0, 7, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 3, 6, 6, 3, 2, 1, 0, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 7, 2, 7, 8, 2, 2, 8, 1, 1, 8, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 3, 1, 7, 1, 11, 7, 11, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 7, 0, 6, 7, 0, 1, 6, 6, 1, 11, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 7, 3, 0, 9, 7, 9, 6, 7, 9, 11, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 9, 11, 7, 8, 11, 6, 7, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 10, 4, 4, 10, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 4, 0, 6, 0, 3, 6, 3, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 4, 10, 10, 4, 8, 0, 9, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 6, 4, 9, 1, 6, 1, 10, 6, 1, 3, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 10, 4, 4, 10, 6, 11, 2, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 1, 11, 10, 0, 3, 10, 6, 0, 0, 6, 4, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 10, 6, 4, 8, 10, 9, 11, 2, 9, 2, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 3, 4, 3, 6, 4, 3, 10, 6, 11, 3, 9, 2, 3, 11, -1, -1, -1, -1}}, 
		{{2, 6, 4, 2, 4, 8, 2, 8, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 4, 2, 2, 4, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 9, 1, 3, 4, 8, 3, 2, 4, 4, 2, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 6, 4, 1, 2, 4, 9, 1, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 11, 3, 11, 6, 3, 3, 6, 8, 8, 6, 4, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 4, 0, 11, 6, 0, 1, 11, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 3, 11, 3, 9, 11, 3, 0, 9, 4, 3, 6, 8, 3, 4, -1, -1, -1, -1}}, 
		{{9, 6, 4, 11, 6, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 5, 9, 6, 7, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 8, 0, 5, 9, 4, 6, 7, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 4, 1, 1, 4, 5, 6, 7, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 6, 4, 3, 8, 4, 5, 3, 3, 5, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 11, 2, 7, 10, 6, 4, 5, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 3, 8, 1, 11, 2, 9, 4, 5, 7, 10, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{6, 7, 10, 11, 4, 5, 11, 2, 4, 4, 2, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 6, 8, 4, 5, 3, 8, 5, 11, 3, 5, 2, 3, 11, -1, -1, -1, -1}}, 
		{{2, 6, 3, 3, 6, 7, 4, 5, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 5, 9, 8, 6, 7, 8, 0, 6, 6, 0, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 1, 0, 4, 5, 1, 7, 3, 2, 7, 2, 6, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 8, 1, 8, 5, 1, 8, 4, 5, 6, 8, 2, 7, 8, 6, -1, -1, -1, -1}}, 
		{{5, 9, 4, 6, 1, 11, 6, 7, 1, 1, 7, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 5, 9, 7, 8, 0, 6, 7, 0, 1, 6, 0, 11, 6, 1, -1, -1, -1, -1}}, 
		{{0, 11, 3, 11, 7, 3, 11, 6, 7, 4, 11, 0, 5, 11, 4, -1, -1, -1, -1}}, 
		{{6, 7, 8, 11, 6, 8, 11, 8, 4, 11, 4, 5, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 8, 10, 9, 10, 6, 9, 6, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 9, 6, 9, 10, 6, 9, 0, 10, 10, 0, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 6, 8, 6, 5, 8, 8, 5, 0, 0, 5, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 1, 3, 6, 5, 3, 10, 6, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 2, 1, 5, 10, 6, 5, 9, 10, 10, 9, 8, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 2, 1, 6, 5, 9, 10, 6, 9, 0, 10, 9, 3, 10, 0, -1, -1, -1, -1}}, 
		{{8, 5, 0, 5, 2, 0, 5, 11, 2, 10, 5, 8, 6, 5, 10, -1, -1, -1, -1}}, 
		{{10, 6, 5, 3, 10, 5, 3, 5, 11, 3, 11, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 9, 8, 3, 2, 9, 2, 5, 9, 2, 6, 5, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 2, 6, 9, 0, 6, 5, 9, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 8, 6, 8, 2, 6, 8, 3, 2, 1, 8, 5, 0, 8, 1, -1, -1, -1, -1}}, 
		{{5, 2, 6, 1, 2, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 6, 8, 6, 9, 8, 6, 5, 9, 1, 6, 3, 11, 6, 1, -1, -1, -1, -1}}, 
		{{1, 11, 6, 0, 1, 6, 0, 6, 5, 0, 5, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 0, 8, 6, 5, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 11, 6, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 11, 7, 7, 11, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 7, 11, 11, 7, 10, 3, 8, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 11, 7, 7, 11, 5, 9, 1, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 7, 10, 11, 5, 7, 1, 3, 8, 1, 8, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 5, 7, 1, 7, 10, 1, 10, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 8, 0, 2, 7, 10, 2, 1, 7, 7, 1, 5, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 10, 5, 10, 2, 5, 5, 2, 9, 9, 2, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 2, 9, 2, 8, 9, 2, 3, 8, 7, 2, 5, 10, 2, 7, -1, -1, -1, -1}}, 
		{{5, 7, 3, 5, 3, 2, 5, 2, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 0, 2, 11, 5, 0, 5, 8, 0, 5, 7, 8, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 0, 9, 11, 3, 2, 11, 5, 3, 3, 5, 7, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 2, 7, 2, 5, 7, 2, 11, 5, 9, 2, 8, 1, 2, 9, -1, -1, -1, -1}}, 
		{{7, 3, 5, 5, 3, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 5, 7, 0, 1, 7, 8, 0, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{5, 7, 3, 9, 5, 3, 0, 9, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 5, 7, 9, 5, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 10, 11, 8, 11, 5, 8, 5, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 3, 4, 3, 10, 4, 4, 10, 5, 5, 10, 11, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 1, 0, 4, 11, 5, 4, 8, 11, 11, 8, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 4, 3, 4, 1, 3, 4, 9, 1, 11, 4, 10, 5, 4, 11, -1, -1, -1, -1}}, 
		{{4, 1, 5, 4, 8, 1, 8, 2, 1, 8, 10, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 10, 5, 10, 1, 5, 10, 2, 1, 0, 10, 4, 3, 10, 0, -1, -1, -1, -1}}, 
		{{2, 5, 10, 5, 8, 10, 5, 4, 8, 0, 5, 2, 9, 5, 0, -1, -1, -1, -1}}, 
		{{4, 9, 5, 10, 2, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 8, 5, 8, 11, 5, 8, 3, 11, 11, 3, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 0, 2, 5, 4, 2, 11, 5, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 1, 0, 5, 4, 8, 11, 5, 8, 3, 11, 8, 2, 11, 3, -1, -1, -1, -1}}, 
		{{11, 5, 4, 2, 11, 4, 2, 4, 9, 2, 9, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 1, 5, 8, 3, 5, 4, 8, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 1, 5, 0, 1, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 8, 3, 5, 4, 3, 5, 3, 0, 5, 0, 9, -1, -1, -1, -1, -1, -1, -1}}, 
		{{4, 9, 5, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 11, 9, 10, 9, 4, 10, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 3, 7, 9, 4, 7, 10, 9, 9, 10, 11, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 0, 4, 7, 10, 0, 10, 1, 0, 10, 11, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 4, 11, 4, 10, 11, 4, 7, 10, 3, 4, 1, 8, 4, 3, -1, -1, -1, -1}}, 
		{{7, 10, 4, 10, 9, 4, 10, 2, 9, 9, 2, 1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 0, 3, 4, 7, 10, 9, 4, 10, 2, 9, 10, 1, 9, 2, -1, -1, -1, -1}}, 
		{{2, 0, 4, 10, 2, 4, 7, 10, 4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 8, 4, 2, 3, 4, 2, 4, 7, 2, 7, 10, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 4, 11, 4, 7, 11, 11, 7, 2, 2, 7, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 7, 2, 7, 0, 2, 7, 8, 0, 9, 7, 11, 4, 7, 9, -1, -1, -1, -1}}, 
		{{7, 11, 4, 11, 0, 4, 11, 1, 0, 3, 11, 7, 2, 11, 3, -1, -1, -1, -1}}, 
		{{8, 4, 7, 2, 11, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{7, 3, 1, 4, 7, 1, 9, 4, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 4, 7, 1, 9, 7, 1, 7, 8, 1, 8, 0, -1, -1, -1, -1, -1, -1, -1}}, 
		{{0, 7, 3, 4, 7, 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 4, 7, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 8, 11, 11, 8, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{10, 11, 9, 3, 10, 9, 0, 3, 9, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 10, 11, 0, 8, 11, 1, 0, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 10, 11, 3, 10, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 8, 10, 1, 9, 10, 2, 1, 10, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{2, 1, 9, 10, 2, 9, 10, 9, 0, 10, 0, 3, -1, -1, -1, -1, -1, -1, -1}}, 
		{{8, 2, 0, 10, 2, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 10, 2, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 9, 8, 2, 11, 8, 3, 2, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{11, 0, 2, 9, 0, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 0, 8, 11, 1, 8, 11, 8, 3, 11, 3, 2, -1, -1, -1, -1, -1, -1, -1}}, 
		{{1, 2, 11, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 9, 8, 1, 9, 3, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{9, 0, 1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{3, 0, 8, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}, 
		{{-1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1}}
	};


	template <typename T, class OutputIterator1, class OutputIterator2>
	void marchingCubes(const Image<T,3>& phi, T value, OutputIterator1 points, OutputIterator2 normals, const Affine<T,3>& affine = Affine<T,3>::Identity())
	{
		// Matrice de transformation de la normale et calcul du gradient de phi
		FMatrix<T,3,3> affineN = affine.matrix().transpose();
		const T sigma = 0.5;
		Image<T,3>
			phix = phi.clone().deriche(sigma,1,0),
			phiy = phi.clone().deriche(sigma,1,1),
			phiz = phi.clone().deriche(sigma,1,2);

		// Valeurs aux coins du cube
		T v[8];

		// Increments pour acceder aux voisins
		const int dx = phi.stride(0);
		const int dy = phi.stride(1);
		const int dz = phi.stride(2);

		// On parcourt le volume
		for (int z=0;z<phi.depth()-1;z++) for (int y=0;y<phi.height()-1;y++) for (int x=0;x<phi.width()-1;x++) {

			// On calcule les valeurs aux 8 coins du cube
			int n = phi.offset(x,y,z);
			v[0] = phi[n];
			v[1] = phi[n+dx];
			v[2] = phi[n+dx+dy];
			v[3] = phi[n+dy];
			v[4] = phi[n+dz];
			v[5] = phi[n+dx+dz];
			v[6] = phi[n+dx+dy+dz];
			v[7] = phi[n+dy+dz];

			// On calcule le numero du cas
			int index = 0;
			for (int i=0;i<8;i++) if (v[i]<value) index |= 1<<i;

			// On traite separement le cas 0 et 255 tres frequent
			if (index == 0 || index == 255) continue;

			// On va chercher le cas dans la table
			const int *cas = MC_CASES_TABLE[index].edges;

			// Pour chaque triangle du cas
			for (;cas[0]>-1;cas+=3) {
				// Pour chaque point du triangle
				for (int edge=0;edge<3;edge++) {
					const int edge_id = cas[edge];
					const int *pts = MC_EDGES_TABLE[edge_id];
					// Indice entre 0 et 7 des points a interpoler 
					const int pt1 = pts[0];
					const int pt2 = pts[1];
					// Coordonnee barycentrique
					const T f = (value - v[pt1]) / (v[pt2] - v[pt1]);
					// Coordonnees du point
					T xx,yy,zz;
					switch (edge_id) {
	case 0 : xx = T(x) + f; yy = T(y); zz = T(z); break;
	case 1 : xx = T(x) + 1; yy = T(y) + f; zz = T(z); break;
	case 2 : xx = T(x) + f; yy = T(y) + 1; zz = T(z); break;
	case 3 : xx = T(x); yy = T(y) + f; zz = T(z); break;
	case 4 : xx = T(x) + f; yy = T(y); zz = T(z) + 1; break;
	case 5 : xx = T(x) + 1; yy = T(y) + f; zz = T(z) + 1; break;
	case 6 : xx = T(x) + f; yy = T(y) + 1; zz = T(z) + 1; break;
	case 7 : xx = T(x); yy = T(y) + f; zz = T(z) + 1; break;
	case 8 : xx = T(x); yy = T(y); zz = T(z) + f; break;
	case 9 : xx = T(x) + 1; yy = T(y); zz = T(z) + f; break;
	case 10 : xx = T(x); yy = T(y) + 1; zz = T(z) + f; break;
	case 11 : xx = T(x) + 1; yy = T(y) + 1; zz = T(z) + f; break;
	default : assert(false);
					}

					// On rajoute le point dans le tableau
					FVector<T,3> p,n;
					affine.apply(xx,yy,zz,p.x(),p.y(),p.z());
					T nx = T(phix.interpolate(xx,yy,zz)), ny = T(phiy.interpolate(xx,yy,zz)), nz = T(phiz.interpolate(xx,yy,zz));
					n = affineN * FVector<T,3>(nx,ny,nz);
					*points++ = p;
					*normals++ = normalize(n);
				}
			}
		}
	}
	///@}
} 
