#include <iostream>

// To use LevelSets with old Imagine++ version
#define IMAGINE_COMPAT_30 
#define mean_curvature_motion(a,b) meanCurvatureMotion(a,b)

#include <Imagine/Images.h>
#include "Imagine/LevelSets/LevelSet.h"

using namespace Imagine;
using namespace std;

void sphDist(Image<float,2>&phi,int x0,int y0,int r) {
	for (int j=0;j<phi.height();j++)
		for (int i=0;i<phi.width();i++){
			int x=abs(i-x0),y=abs(j-y0);
			phi(i,j)=float(sqrt(double(x*x+y*y))-r);
		}
}

void rectDist(Image<float,2>&phi,int x0,int y0,int a,int b) {
	// Question 1: 
	//    programmer la distance  un rectangle centr en (x0,y0) et de dimensions (2a,2b)
	//    utiliser rectDist dans level_set()
}


void level_set(int type)
{
	// Load level set function
	int w=128,h=96;
	int zm=4;
	Image<float,2> phi(w,h);
	sphDist(phi,w/2,h/2,w/5);
	//rectDist(phi,w/2,h/2,w/4,h/4);
	
	// For display purposes
	Window W=OpenWindow(zm*w,zm*h);
	display(grey(enlarge(phi,zm)));
	display_zero_level(phi,Red,Affine<float,2>(Scaling(),float(zm)));
	Click();

	// Narrow band
	const float thickness1 = 6.0f;
	const float thickness2 = 3.0f;
	NarrowBand<float,2> nb(phi);

	// Time integrator
	Euler<float,2> integr(phi);

	// Initialization of the narrow band
	nb.init<Weno3>(thickness1,integr);

	// Main loop
	const int niters = 10000;
	const float dt = 0.05f;
	for (int t=0;t<niters;t++) {
		if (t%50==0) {
			Clear();
			display_zero_level(phi,Red,Affine<float,2>(Scaling(),float(zm)));
		}
		// Reinitialize if needed
		if (nb.must_reinit(thickness2)) {
			nb.reinit<Weno3>(thickness1,integr);
		}
		for (NarrowBand<float,2>::iterator p = nb.begin() ; p != nb.end() ; ++p) {
			switch (type) {
			case 0:
				integr(*p) = mean_curvature_motion(phi,*p);
				break;
			case 1:
				integr(*p) = normal_motion<Weno3>(phi, *p, -.1f);
				break;
			case 2:
				integr(*p) = normal_motion<Weno3>(phi, *p, .1f);
				break;
			case 3:
				{
					FVector<float,2> v( .1f, 0.f );
					integr(*p) = advection<Weno3>(phi, *p, v);
				}
				break;
			}
		}
		// Time integration
		integr.step(nb.begin(),nb.end(),dt);
	}
	Click();
	CloseWindow(W);
}

RGB<int> meanCol(const Image<Color,2>& I) {
	int u,v;
	GetMouse(u,v);
	int n=10;
	// Question 4:
	//   calculer et retourner la valeur moyenne de I autour de (u,v) +/- n
	RGB<int> M(0);
	return M;
}

void activeContours(string name,int zm,float alpha,float C1,float C2,float dt,float tf,bool regions=false,float C3=0)
{
	Image<Color,2> I;
	load(I,strSRCPATH(name));
	int w=I.width(),h=I.height();
	Window W=OpenWindow(zm*w,zm*h);
	display(I);
	RGB<int> Min,Mout; // int et non byte: essentiel pour faire I(p)-inCol 
	if (regions) {
		cout << "Click an inside point" << endl;
		Min=meanCol(I);
		cout << "Click an outside  point" << endl;
		Mout=meanCol(I);
		Click();
	}

	// Question 2:
	//   dcommenter le premier appel  activeContours
	//   calculer et afficher G=1/(1+C1*|grad(I)|^2) en estimant les drives de I avec deriche() de paramtre alpha

	// Question 3:
	//   faire un level set avec une vitesse somme de:
	//       mean_curvature : multipliee par G
	//       advection: de force -grad(G)
	//       normal_motion: -C2*G

	// Question 5:
	//   dcommenter les autres appels a activeContours
	//   dans le cas region=true, remplacer le normal_motion par C3*(|I-Mout|^2-|I-Min|^2)
	Click();
	CloseWindow(W);
}


int main(int argc, char **argv)
{
	for (int type=0;type<4;type++)
		level_set(type);
	
	// activeContours("birds.jpg",1,2.f,10.f,.2f,.5f,1400);
	// activeContours("fishes.jpg",1,1.f,1.f,1.f,.2f,350.f);
	// activeContours("fishes.jpg",1,1.f,1.f,0.f,.2f,1500.f,true,1e-6f);
	Terminate();
	return 0;
}
